@@ -2,7 +2,7 @@
  * GTimer
  *
  * Copyright:
- *	(C) 1999 Craig Knudsen, cknudsen@radix.net
+ *	(C) 1999 Craig Knudsen, cknudsen@cknudsen.com
  *	See accompanying file "COPYING".
  * 
  *	This program is free software; you can redistribute it and/or
@@ -24,12 +24,17 @@
  *	Helps you keep track of time spent on different tasks.
  *
  * Author:
- *	Craig Knudsen, cknudsen@radix.net, http://www.radix.net/~cknudsen
+ *	Craig Knudsen, cknudsen@cknudsen.com, http://www.cknudsen.com
  *
  * Home Page:
- *	http://www.radix.net/~cknudsen/gtimer/
+ *	http://www.cknudsen.com/gtimer/
  *
  * History:
+ *	09-Mar-2003	Added support for -noversioncheck option
+ *	06-Mar-2003	Added support for -resume option which will start
+ *			timing the same task that was being timed last
+ *			time gtimer exited.
+ *	01-Mar-2003	Added projects
  *	09-Mar-2000	Improved idle handling.  On revert, the tasks are
  *			no longer reloaded from the data files and the
  *			display will not be resorted.
@@ -148,6 +153,7 @@
 #include <gdk/gdkx.h>
 #endif
 
+#include "project.h"
 #include "task.h"
 #include "gtimer.h"
 #include "config.h"
@@ -244,8 +250,9 @@ int num_timing = 0;
 TaskData **visible_tasks; /* not hidden */
 int num_visible_tasks;
 
-list_column_def task_list_columns[3] = {
-  { "Task",	230, 0,	GTK_JUSTIFY_LEFT,	(gboolean)1,	NULL },
+list_column_def task_list_columns[4] = {
+  { "Project",	150, 0,	GTK_JUSTIFY_LEFT,	(gboolean)1,	NULL },
+  { "Task",	150, 0,	GTK_JUSTIFY_LEFT,	(gboolean)1,	NULL },
   { "Today",	70, 70,	GTK_JUSTIFY_RIGHT,	(gboolean)0,	NULL },
   { "Total",	70, 70,	GTK_JUSTIFY_RIGHT,	(gboolean)0,	NULL }
 };
@@ -263,13 +270,15 @@ static void start_callback ( GtkWidget *widget, gpointer data );
 static void switch_to_callback ( GtkWidget *widget, gpointer data );
 static void stop_callback ( GtkWidget *widget, gpointer data );
 static void stop_all_callback ( GtkWidget *widget, gpointer data );
-static void add_callback ( GtkWidget *widget, gpointer data );
-static void edit_callback ( GtkWidget *widget, gpointer data );
-static void hide_callback ( GtkWidget *widget, gpointer data );
-static void unhide_callback ( GtkWidget *widget, gpointer data );
-static void delete_callback ( GtkWidget *widget, gpointer data );
+static void task_add_callback ( GtkWidget *widget, gpointer data );
+static void task_edit_callback ( GtkWidget *widget, gpointer data );
+static void task_hide_callback ( GtkWidget *widget, gpointer data );
+static void task_unhide_callback ( GtkWidget *widget, gpointer data );
+static void task_delete_callback ( GtkWidget *widget, gpointer data );
 static void increment_time_callback ( GtkWidget *widget, gpointer data );
 static void decrement_time_callback ( GtkWidget *widget, gpointer data );
+static void project_add_callback ( GtkWidget *widget, gpointer data );
+static void project_edit_callback ( GtkWidget *widget, gpointer data );
 static void report_callback ( GtkWidget *widget, gpointer data );
 static void annotate_callback ( GtkWidget *widget, gpointer data );
 static void idle_reset_callback ( GtkWidget *widget, gpointer data );
@@ -349,12 +358,12 @@ TTPulldown task_menu[] = {
   { LABEL_STOP_TIMING, stop_callback, NULL, 0, 0 },
   { LABEL_STOP_ALL_TIMING, stop_all_callback, NULL, 'x', GDK_CONTROL_MASK },
   { 0, NULL, NULL, 0, 0 },
-  { LABEL_NEW, add_callback, NULL, 'n', GDK_CONTROL_MASK },
-  { LABEL_EDIT, edit_callback, NULL, 'e', GDK_CONTROL_MASK },
+  { LABEL_NEW, task_add_callback, NULL, 'n', GDK_CONTROL_MASK },
+  { LABEL_EDIT, task_edit_callback, NULL, 'e', GDK_CONTROL_MASK },
   { LABEL_ANNOTATE, annotate_callback, NULL, 'a', GDK_CONTROL_MASK },
-  { LABEL_HIDE, hide_callback, NULL, 'h', GDK_CONTROL_MASK },
-  { LABEL_UNHIDE, unhide_callback, NULL, 'u', GDK_CONTROL_MASK },
-  { LABEL_DELETE, delete_callback, NULL, 0, 0 },
+  { LABEL_HIDE, task_hide_callback, NULL, 'h', GDK_CONTROL_MASK },
+  { LABEL_UNHIDE, task_unhide_callback, NULL, 'u', GDK_CONTROL_MASK },
+  { LABEL_DELETE, task_delete_callback, NULL, 0, 0 },
   { 0, NULL, NULL, 0, 0 },
   { LABEL_INCREMENT_5, increment_time_callback, (gpointer) (5*60), 'i',
     GDK_CONTROL_MASK },
@@ -365,7 +374,13 @@ TTPulldown task_menu[] = {
   { LABEL_SET_TO_ZERO, increment_time_callback, (gpointer) 0, 0, 0 },
   { -1, NULL, NULL, 0, 0 }
 };
-/* Task pulldown menu */
+/* Project pulldown menu */
+TTPulldown project_menu[] = {
+  { LABEL_NEW, project_add_callback, NULL, 'n', GDK_CONTROL_MASK },
+  { LABEL_EDIT, project_edit_callback, NULL, 'e', GDK_CONTROL_MASK },
+  { -1, NULL, NULL, 0, 0 }
+};
+/* Report pulldown menu */
 TTPulldown report_menu[] = {
   { LABEL_DAILY, report_callback, (gpointer)REPORT_TYPE_DAILY, 0, 0 },
   { LABEL_WEEKLY, report_callback, (gpointer)REPORT_TYPE_WEEKLY, 0, 0 },
@@ -409,9 +424,9 @@ TTToolButton main_toolbar[] = {
   { "Annotate", TOOLTIP_ANNOTATE, annotate_xpm,
     annotate_callback, NULL, NULL },
   { "Add", TOOLTIP_ADD, new_xpm,
-    add_callback, NULL, NULL },
+    task_add_callback, NULL, NULL },
   { "Edit", TOOLTIP_EDIT, edit_xpm,
-    edit_callback, NULL, NULL },
+    task_edit_callback, NULL, NULL },
   { NULL, 0, NULL, NULL, NULL, NULL }
 };
 
@@ -509,6 +524,29 @@ int ind;
 
 
 
+static int sort_task_by_project ( td1, td2 )
+TaskData **td1;
+TaskData **td2;
+{
+  TaskData *tda = *td1;
+  TaskData *tdb = *td2;
+  int ret;
+
+  /* put tasks with no projects (-1) at the end of the list, most
+     recent projects at the top */
+  if ( tda->task->project_id > tdb->task->project_id )
+    ret = -1;
+  else if ( tda->task->project_id < tdb->task->project_id )
+    ret = 1;
+  else
+    ret = 0;
+  
+  if ( sort_forward )
+    return ( ret );
+  else
+    return ( - ret );
+}
+
 static int sort_task_by_name ( td1, td2 )
 TaskData **td1;
 TaskData **td2;
@@ -599,6 +637,7 @@ void save_all ()
 {
   update_tasks ();
   taskSaveAll ( taskdir );
+  projectSaveAll ( taskdir );
   time ( &last_save );
   modified_since_save = 0;
 }
@@ -624,6 +663,8 @@ GtkWidget *widget;
 gpointer data;
 {
   gint w, h;
+  char temp[128];
+  int loop;
 
   /* save task data */
   save_all ();
@@ -635,12 +676,27 @@ gpointer data;
 
   /* get columns widths in main window */
   w = GTK_CLIST ( task_list )->column[0].width;
-  configSetAttributeInt ( CONFIG_MAIN_WINDOW_TASK_WIDTH, w );
+  configSetAttributeInt ( CONFIG_MAIN_WINDOW_PROJECT_WIDTH, w );
   w = GTK_CLIST ( task_list )->column[1].width;
-  configSetAttributeInt ( CONFIG_MAIN_WINDOW_TODAY_WIDTH, w );
+  configSetAttributeInt ( CONFIG_MAIN_WINDOW_TASK_WIDTH, w );
   w = GTK_CLIST ( task_list )->column[2].width;
+  configSetAttributeInt ( CONFIG_MAIN_WINDOW_TODAY_WIDTH, w );
+  w = GTK_CLIST ( task_list )->column[3].width;
   configSetAttributeInt ( CONFIG_MAIN_WINDOW_TOTAL_WIDTH, w );
 
+  /* keep track of which tasks were being timed in case the user starts up
+     with -resume next time */
+  temp[0] = '\0';
+  for ( loop = 0; loop < num_visible_tasks; loop++ ) {
+    if ( visible_tasks[loop]->timer_on ) {
+      if ( strlen ( temp ) )
+        strcat ( temp, "," );
+      sprintf ( temp + strlen ( temp ), "%d",
+        visible_tasks[loop]->task->number );
+    }
+  }
+  configSetAttribute ( CONFIG_LAST_TIMED_TASKS, temp );
+
   /* save config settings */
   configSaveAttributes ( config_file );
 
@@ -698,7 +754,7 @@ gpointer data;
     gtk_major_version, gtk_minor_version, gtk_micro_version );
   strcat ( text, gettext("Author") );
   sprintf ( text + strlen ( text ),
-     ":\nCraig Knudsen\ncknudsen@radix.net\n\n" );
+     ":\nCraig Knudsen\ncknudsen@cknudsen.com\n\n" );
   create_confirm_window ( CONFIRM_ABOUT,
     gettext("About"),
     text,
@@ -714,14 +770,14 @@ gpointer data;
 
 
 
-static void add_callback ( widget, data )
+static void task_add_callback ( widget, data )
 GtkWidget *widget;
 gpointer data;
 {
-  create_edit_window ( NULL );
+  create_task_edit_window ( NULL );
 }
 
-static void edit_callback ( widget, data )
+static void task_edit_callback ( widget, data )
 GtkWidget *widget;
 gpointer data;
 {
@@ -735,13 +791,13 @@ gpointer data;
       NULL, NULL, NULL,
       NULL );
   } else {
-    create_edit_window ( visible_tasks[st] );
+    create_task_edit_window ( visible_tasks[st] );
   }
 }
 
 
 
-static void hide_callback ( widget, data )
+static void task_hide_callback ( widget, data )
 GtkWidget *widget;
 gpointer data;
 {
@@ -774,7 +830,7 @@ gpointer data;
 
 
 
-static void unhide_callback ( widget, data )
+static void task_unhide_callback ( widget, data )
 GtkWidget *widget;
 gpointer data;
 {
@@ -851,7 +907,7 @@ gpointer data;
 }
 
 
-static void delete_callback ( widget, data )
+static void task_delete_callback ( widget, data )
 GtkWidget *widget;
 gpointer data;
 {
@@ -1102,6 +1158,42 @@ gpointer data;
 }
 
 
+static void project_add_callback ( widget, data )
+GtkWidget *widget;
+gpointer data;
+{
+  create_project_edit_window ( NULL );
+}
+
+static void project_edit_callback ( widget, data )
+GtkWidget *widget;
+gpointer data;
+{
+  int st = get_selected_task ();
+  Project *p;
+
+  if ( st < 0 || ! num_visible_tasks ) {
+    create_confirm_window ( CONFIRM_ERROR,
+      gettext("Error"),
+      gettext("You have not selected\na task."),
+      gettext("Ok"), NULL, NULL,
+      NULL, NULL, NULL,
+      NULL );
+  } else {
+    if ( visible_tasks[st]->task->project_id < 0 ) {
+      create_confirm_window ( CONFIRM_ERROR,
+        gettext("Error"),
+        gettext("The selected task does not\nhave a project."),
+        gettext("Ok"), NULL, NULL,
+        NULL, NULL, NULL,
+        NULL );
+    } else {
+      p = projectGet ( visible_tasks[st]->task->project_id );
+      create_project_edit_window ( p );
+    }
+  }
+}
+
 
 static void report_callback ( widget, data )
 GtkWidget *widget;
@@ -1376,8 +1468,8 @@ gpointer data;
   } else {
     gdk_input_id = gdk_input_add ( (gint) connection,
       GDK_INPUT_READ, read_http_socket, NULL );
-    ret = httpGet ( connection, GTIMER_VERSION_CHECK_PATH,
-      NULL, NULL, 0, read_version );
+    ret = httpGet ( connection, GTIMER_VERSION_CHECK_SERVER,
+      GTIMER_VERSION_CHECK_PATH, NULL, NULL, 0, read_version );
     if ( ret ) {
       if ( ! version_check_is_auto ) {
         sprintf ( msg, "%s:\n\n%s", gettext("HTTP Error"),
@@ -1480,14 +1572,18 @@ int col;
   switch ( col ) {
     case 0:
       qsort ( visible_tasks, num_visible_tasks, sizeof ( TaskData * ),
-        sort_task_by_name );
+        sort_task_by_project );
       break;
     case 1:
+      qsort ( visible_tasks, num_visible_tasks, sizeof ( TaskData * ),
+        sort_task_by_name );
+      break;
+    case 2:
       qsort ( visible_tasks, num_visible_tasks, sizeof ( TaskData * ),
         sort_task_by_today );
       break;
     default:
-    case 2:
+    case 3:
       qsort ( visible_tasks, num_visible_tasks, sizeof ( TaskData * ),
         sort_task_by_total );
       break;
@@ -1660,6 +1756,32 @@ static GtkWidget *create_main_window_menu_bar ()
   gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), root_menu);
   gtk_widget_show (root_menu);
 
+  /* Project menu */
+  root_menu = gtk_menu_item_new_with_label ( gettext("Project") );
+  menu = gtk_menu_new ();
+#if GTK_VERSION > 10100
+  menu_item = gtk_tearoff_menu_item_new ();
+  gtk_menu_append ( GTK_MENU ( menu ), menu_item );
+  gtk_widget_show ( menu_item );
+#endif
+  for ( loop = 0; project_menu[loop].label_index >= 0; loop++ ) {
+    if ( project_menu[loop].label_index > 0 )
+      menu_item = gtk_menu_item_new_with_label (
+        get_label_text ( project_menu[loop].label_index ) );
+    else
+      menu_item = gtk_menu_item_new ();
+    if ( project_menu[loop].callback )
+      gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+        GTK_SIGNAL_FUNC ( project_menu[loop].callback ),
+        (gpointer) project_menu[loop].data );
+    gtk_menu_append ( GTK_MENU ( menu ), menu_item );
+    gtk_widget_show ( menu_item );
+  }
+
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_menu), menu);
+  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), root_menu);
+  gtk_widget_show (root_menu);
+
   /* Report menu */
   root_menu = gtk_menu_item_new_with_label ( gettext("Report") );
   menu = gtk_menu_new ();
@@ -1806,9 +1928,12 @@ void update_list () {
   time_t now, diff, total, today;
   GdkPixmap *icon;
   GdkBitmap *mask;
-  char *row[3];
+  char *row[4];
   int total_today = 0;
   char today_test[20];
+  char *project_name;
+  Project *p;
+  static char *noproject = "";
 
   time ( &now );
   if ( config_animate_enabled ) {
@@ -1826,12 +1951,19 @@ void update_list () {
     if ( taskdata->new_task ) {
       modified_since_save = 1;
       taskdata->new_task = 0;
-      row[0] = taskdata->task->name;
-      row[1] = "00:00:00";
+      if ( taskdata->task->project_id > 0 ) {
+        p = projectGet ( taskdata->task->project_id );
+        project_name = p->name;
+      } else {
+        project_name = noproject;
+      }
+      row[0] = project_name;
+      row[1] = taskdata->task->name;
       row[2] = "00:00:00";
+      row[3] = "00:00:00";
       gtk_clist_append ( GTK_CLIST(task_list), row );
       gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0, 
-        taskdata->task->name, 2, blankicon, blankicon_mask);
+        taskdata->project_name, 2, blankicon, blankicon_mask);
       continue;
     }
     /* update the name ? */
@@ -1840,14 +1972,15 @@ void update_list () {
       taskdata->name_updated = 0;
       if ( taskdata->timer_on ) {
         gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0,
-          taskdata->task->name, 2, icon, mask);
+          taskdata->project_name, 2, icon, mask);
         taskdata->last_on = 1;
       } else {
         gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0, 
-          taskdata->task->name, 2, blankicon, blankicon_mask);
+          taskdata->project_name, 2, blankicon, blankicon_mask);
         taskdata->last_on = 0;
       }
     }
+    gtk_clist_set_text ( GTK_CLIST(task_list), i, 1, taskdata->task->name );
     /* calc total */
     total = taskdata->total;
     if ( taskdata->todays_entry )
@@ -1862,7 +1995,7 @@ void update_list () {
     s = total % 60;
     sprintf ( text, "%d:%02d:%02d", h, m, s );
     if ( strcmp ( text, taskdata->last_total ) || taskdata->moved ) {
-      gtk_clist_set_text ( GTK_CLIST(task_list), i, 2, text );
+      gtk_clist_set_text ( GTK_CLIST(task_list), i, 3, text );
       strcpy ( taskdata->last_total, text );
     }
     taskdata->last_total_int = total;
@@ -1879,7 +2012,7 @@ void update_list () {
     s = today % 60;
     sprintf ( text, "%d:%02d:%02d", h, m, s );
     if ( strcmp ( text, taskdata->last_today ) || taskdata->moved ) {
-      gtk_clist_set_text ( GTK_CLIST(task_list), i, 1, text );
+      gtk_clist_set_text ( GTK_CLIST(task_list), i, 2, text );
       strcpy ( taskdata->last_today, text );
     }
     taskdata->last_today_int = today;
@@ -1887,11 +2020,11 @@ void update_list () {
     if ( taskdata->timer_on ) {
       modified_since_save = 1;
       gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0,
-        taskdata->task->name, 2, icon, mask);
+        taskdata->project_name, 2, icon, mask);
       taskdata->last_on = 1;
     } else if ( ! taskdata->timer_on && taskdata->last_on ) {
       gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0, 
-        taskdata->task->name, 2, blankicon, blankicon_mask);
+        taskdata->project_name, 2, blankicon, blankicon_mask);
       taskdata->last_on = 0;
     }
     taskdata->moved = 0;
@@ -1916,14 +2049,17 @@ void update_list () {
 static void build_list () {
   Task *task;
   TaskData *taskdata;
+  Project *p;
+  char *project_name;
   char today_str[100], total_str[100];
-  char *row[3];
+  char *row[4];
   int i, j;
   GdkPixmap *icon;
   GdkBitmap *mask;
   time_t now;
   static int first = 1;
   GtkWidget *win;
+  static char *noproject = "";
 
   if ( splash_window )
     win = splash_window;
@@ -1980,6 +2116,12 @@ static void build_list () {
       }
       strcpy ( taskdata->last_today, "" );
       strcpy ( taskdata->last_total, "" );
+      taskdata->project_name = "";
+      if ( taskdata->task->project_id >= 0 ) {
+        p = projectGet ( task->project_id );
+        if ( p != NULL )
+          taskdata->project_name = p->name;
+      }
       tasks[num_tasks++] = taskdata;
       if ( ! taskOptionEnabled ( taskdata->task, GTIMER_TASK_OPTION_HIDDEN ) )
         visible_tasks[num_visible_tasks++] = taskdata;
@@ -1996,18 +2138,23 @@ static void build_list () {
   for ( i = 0; i < num_visible_tasks; i++ ) {
     visible_tasks[i]->moved = 1;
     task = visible_tasks[i]->task;
-    row[0] = task->name;
+    row[0] = noproject;
+    if ( task->project_id >= 0 ) {
+      p = projectGet ( task->project_id );
+      row[0] = p->name;
+    }
+    row[1] = task->name;
     sprintf ( today_str, "00:00:00" );
-    row[1] = today_str;
+    row[2] = today_str;
     sprintf ( total_str, "00:00:00" );
-    row[2] = total_str;
+    row[3] = total_str;
     gtk_clist_append ( GTK_CLIST(task_list), row );
     if ( tasks[i]->timer_on )
       gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0, 
-        task->name, 2, icon, mask);
+        visible_tasks[i]->project_name, 2, icon, mask);
     else
       gtk_clist_set_pixtext (GTK_CLIST (task_list), i, 0, 
-        task->name, 2, blankicon, blankicon_mask);
+        visible_tasks[i]->project_name, 2, blankicon, blankicon_mask);
   }
   /* gtk_clist_thaw ( GTK_CLIST(task_list) ); */
 
@@ -2297,13 +2444,15 @@ void create_main_window () {
     gtk_widget_show ( toolbar );
 
   /* add in list here */
-  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_TASK_WIDTH, &w ) == 0 )
+  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_PROJECT_WIDTH, &w ) == 0 )
     task_list_columns[0].width = w;
-  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_TODAY_WIDTH, &w ) == 0 )
+  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_TASK_WIDTH, &w ) == 0 )
     task_list_columns[1].width = w;
-  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_TOTAL_WIDTH, &w ) == 0 )
+  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_TODAY_WIDTH, &w ) == 0 )
     task_list_columns[2].width = w;
-  task_list = create_list_column_def ( 3, task_list_columns );
+  if ( configGetAttributeInt ( CONFIG_MAIN_WINDOW_TOTAL_WIDTH, &w ) == 0 )
+    task_list_columns[3].width = w;
+  task_list = create_list_column_def ( 4, task_list_columns );
   gtk_clist_set_selection_mode (GTK_CLIST (task_list), GTK_SELECTION_BROWSE);
   gtk_widget_set_usize (GTK_WIDGET (task_list), 350, 150);
 #if GTK_VERSION < 10100
@@ -2397,10 +2546,12 @@ int main ( int argc, char *argv[] ) {
 #endif
   time_t now;
   struct tm *tm;
-  int loop, loop2, offset;
-  char *ptr;
+  int loop, loop2, offset, lastTaskNumber;
+  char *ptr, *ptr2;
   struct stat buf;
   int display_splash = 1;
+  int resume = 0; /* start time tasks from last exit */
+  int noversioncheck = 0; /* don't check for new version */
   GtkWidget *win;
   int w = 0, h = 0;
 #ifdef HAVE_LIBINTL_H
@@ -2483,9 +2634,21 @@ int main ( int argc, char *argv[] ) {
       sprintf ( localedir, "%s/locale", taskdir );
       bindtextdomain ( "gtimer", localedir );
       free ( localedir );
+#endif
+#if GTK_VERSION > 10100
+      if ( gtkrc )
+        free ( gtkrc );
+      gtkrc = (char *) malloc ( strlen ( taskdir ) +
+        strlen ( "gtkrc" ) + 2 );
+      sprintf ( gtkrc, "%s/%s", taskdir, "gtkrc" );
+      gtk_rc_parse ( gtkrc );
 #endif
     } else if ( strcmp ( argv[loop], "-nosplash" ) == 0 ) {
       display_splash = 0;
+    } else if ( strcmp ( argv[loop], "-resume" ) == 0 ) {
+      resume = 1;
+    } else if ( strcmp ( argv[loop], "-noversioncheck" ) == 0 ) {
+      noversioncheck = 1;
     } else if ( strcmp ( argv[loop], "-midnight" ) == 0 ) {
       if ( ! argv[loop+1] ) {
         fprintf ( stderr, "%s: -midnight %s.\n",
@@ -2561,6 +2724,9 @@ int main ( int argc, char *argv[] ) {
   */
   configSetAttribute ( CONFIG_VERSION, GTIMER_VERSION );
 
+  /* load all projects */
+  projectLoadAll ( taskdir );
+
   /* load all tasks */
   taskLoadAll ( taskdir );
 
@@ -2631,7 +2797,7 @@ int main ( int argc, char *argv[] ) {
   }
 
   /* handle tasks specified with -start */
-  for ( loop = 0; loop < nmatches; loop++ ) {
+  for ( loop = 0; ! resume && loop < nmatches; loop++ ) {
     found = 0;
     for ( loop2 = 0; loop2 < num_visible_tasks && ! found; loop2++ ) {
       td =  visible_tasks[loop2];
@@ -2657,12 +2823,41 @@ int main ( int argc, char *argv[] ) {
     }
   }
 
+  if ( resume &&
+    ( configGetAttribute ( CONFIG_LAST_TIMED_TASKS, &ptr ) == 0 ) ) {
+    ptr = strdup ( ptr );
+    for ( ptr2 = strtok ( ptr, "," ); ptr2 != NULL;
+      ptr2 = strtok ( NULL, "," ) ) {
+      lastTaskNumber = atoi ( ptr2 );
+      for ( loop = 0; loop < num_visible_tasks; loop++ ) {
+        td =  visible_tasks[loop];
+        if ( td->task->number == lastTaskNumber ) {
+          num_timing++;
+          td->timer_on = 1;
+          time ( &td->on_since );
+          if ( td->todays_entry == NULL )
+            td->todays_entry = taskNewTimeEntry ( td->task, today_year,
+              today_mon, today_mday );
+          /* select the task */
+          gtk_clist_select_row ( GTK_CLIST ( task_list ), loop, 0 );
+          /* make task visible */
+          move_to_task = loop2;
+          if ( ! splash_window )
+            gtk_clist_moveto ( GTK_CLIST ( task_list ), loop, -1, 0.5, 0 );
+          break;
+        }
+      }
+    }
+    free ( ptr );
+  }
+
   /* Add a timeout to update the display once a second */
   gtk_timeout_add ( 1000, timeout_handler, NULL );
 
   /* Add a timeout to check for a new version in 30 seconds
      (We will just check every 30 seconds to see if we should check */
-  gtk_timeout_add ( 30 * 1000, version_timeout_handler, NULL );
+  if ( ! noversioncheck )
+    gtk_timeout_add ( 30 * 1000, version_timeout_handler, NULL );
 
   /* record time for use with autosave */
   time ( &last_save );
