@@ -2,7 +2,7 @@
  * GTimer
  *
  * Copyright:
- *	(C) 1999 Craig Knudsen, cknudsen@radix.net
+ *	(C) 1999 Craig Knudsen, cknudsen@cknudsen.com
  *	See accompanying file "COPYING".
  * 
  *	This program is free software; you can redistribute it and/or
@@ -24,12 +24,14 @@
  *	Helps you keep track of time spent on different tasks.
  *
  * Author:
- *	Craig Knudsen, cknudsen@radix.net, http://www.radix.net/~cknudsen
+ *	Craig Knudsen, cknudsen@cknudsen.com, http://www.cknudsen.com
  *
  * Home Page:
- *	http://www.radix.net/~cknudsen/gtimer/
+ *	http://www.cknudsen.com/gtimer/
  *
  * History:
+ *	03-Mar-2003	Added support for rounding.
+ *	28-Feb-2003	Include project name or "none" in reports
  *	09-Mar-2000	Updated calls to create_confirm_window()
  *	25-Mar-1999	Added use of gtk_window_set_transient_for() for
  *			GTK 1.1 or later.
@@ -102,6 +104,7 @@
 #define gettext(a)      a
 #endif
 
+#include "project.h"
 #include "task.h"
 #include "gtimer.h"
 #include "config.h"
@@ -112,6 +115,20 @@
 
 #define NO_TOOLTIPS	1
 
+/* CSS stylesheet */
+#define CSS_STYLE "\
+<style>\n\
+body {\n\
+  background: #ffffff;\n\
+}\n\
+th {\n\
+  background: #c0c0c0;\n\
+}\n\
+td {\n\
+  background: #ffffff;\n\
+}\n\
+</style>\n"
+
 #define ONE_DAY	(3600*24)
 static int month_days[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 static int lmonth_days[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
@@ -127,10 +144,13 @@ extern GdkPixmap *appicon2_mask;
 #define REPORT_RANGE_TODAY              0
 #define REPORT_RANGE_THIS_WEEK          1
 #define REPORT_RANGE_LAST_WEEK          2
-#define REPORT_RANGE_THIS_MONTH         3
-#define REPORT_RANGE_LAST_MONTH         4
-#define REPORT_RANGE_THIS_YEAR          5
+#define REPORT_RANGE_THIS_AND_LAST_WEEK 3
+#define REPORT_RANGE_LAST_TWO_WEEKS     4
+#define REPORT_RANGE_THIS_MONTH         5
+#define REPORT_RANGE_LAST_MONTH         6
+#define REPORT_RANGE_THIS_YEAR          7
 static char *time_options[] = { "Today", "This Week", "Last Week",
+  "This Week & Last Week", "Last Two Weeks",
   "This Month", "Last Month", "This Year", NULL };
 
 /* Output types */
@@ -146,6 +166,22 @@ static char *output_options[] = { "Text", "HTML", NULL };
 static char *data_options[] = { "Hours worked", "Annotations",
   "Hours & Annotations", NULL };
 
+/* Rounding options */
+#define REPORT_ROUND_NONE		0
+#define REPORT_ROUND_MINUTE		60
+#define REPORT_ROUND_5_MINUTES		300
+#define REPORT_ROUND_10_MINUTES		600
+#define REPORT_ROUND_15_MINUTES		900
+#define REPORT_ROUND_30_MINUTES		1800
+#define REPORT_ROUND_HOUR		3600
+static char *round_options[] = { "None", "Minute", "5 Minutes",
+  "10 Minutes", "15 minutes", "30 Minutes", "Hour", NULL };
+static int round_values[]= { REPORT_ROUND_NONE,
+  REPORT_ROUND_MINUTE, REPORT_ROUND_5_MINUTES,
+  REPORT_ROUND_10_MINUTES, REPORT_ROUND_15_MINUTES,
+  REPORT_ROUND_30_MINUTES, REPORT_ROUND_HOUR, -1 };
+  
+
 typedef struct {
   TaskData *taskdata;
   time_t weekly_total;
@@ -163,11 +199,13 @@ typedef struct {
 typedef struct {
   GtkWidget *window;
   GtkWidget *time_menu;
-  GtkWidget *time_menu_items[6];
+  GtkWidget *time_menu_items[8];
   GtkWidget *output_menu;
   GtkWidget *output_menu_items[2];
   GtkWidget *data_menu;
   GtkWidget *data_menu_items[3];
+  GtkWidget *round_menu;
+  GtkWidget *round_menu_items[8];
   GtkWidget *task_list;
   TaskData **tasks;
   int num_tasks;
@@ -186,9 +224,6 @@ typedef struct {
 } DisplayReportData;
 
 
-/* Local functions */
-static char *get_client_path ( char *file );
-
 
 
 
@@ -426,7 +461,8 @@ gpointer data;
   gdk_window_set_icon ( GTK_WIDGET ( drd->printwin )->window,
     NULL, appicon2, appicon2_mask );
 
-#if GTK_VERSION >= 10100
+#if OLD_GTK
+#else
   gtk_window_set_transient_for ( GTK_WINDOW ( drd->printwin ),
     GTK_WINDOW ( drd->window ) );
 #endif
@@ -475,7 +511,7 @@ char *text;
 {
   DisplayReportData *drd;
   GtkWidget *window, *hbox, *button, *textarea, *vscrollbar;
-  GtkStyle *style;
+  static GdkFont *font = NULL;
   /*GtkTooltips *tooltips;*/
   char msg[100];
 
@@ -495,16 +531,14 @@ char *text;
   gtk_box_pack_start ( GTK_BOX ( GTK_DIALOG (window)->vbox ),
     hbox, TRUE, TRUE, 2 );
 
-  style = gtk_style_new ();
-  gdk_font_unref ( style->font );
-  style->font = gdk_font_load (
-    "-adobe-courier-medium-r-*-*-12-*-*-*-*-*-*-*" );
-  if ( style->font == NULL )
-    style->font = gdk_font_load ( "fixed" );
-  gtk_widget_push_style ( style );
+  if ( font == NULL ) {
+    font = gdk_font_load ( "-adobe-courier-medium-r-*-*-12-*-*-*-*-*-*-*" );
+    if ( font == NULL )
+      font = gdk_font_load ( "fixed" );
+  }
 
   textarea = gtk_text_new ( NULL, NULL );
-  gtk_widget_set_usize ( textarea, 300, 200 );
+  gtk_widget_set_usize ( textarea, 450, 300 );
   gtk_box_pack_start ( GTK_BOX ( hbox ),
     textarea, TRUE, TRUE, 2 );
   gtk_text_set_word_wrap ( GTK_TEXT ( textarea ), 1 );
@@ -512,12 +546,10 @@ char *text;
   gtk_widget_realize ( textarea );
   gtk_text_freeze ( GTK_TEXT ( textarea ) );
   gtk_text_set_point ( GTK_TEXT ( textarea ), 0 );
-  gtk_text_insert ( GTK_TEXT ( textarea ), NULL, NULL, NULL,
+  gtk_text_insert ( GTK_TEXT ( textarea ), font, NULL, NULL,
     text, strlen ( text ) );
   gtk_text_thaw ( GTK_TEXT ( textarea ) );
 
-  gtk_widget_pop_style ();
-
   vscrollbar = gtk_vscrollbar_new ( GTK_TEXT ( textarea )->vadj );
   gtk_box_pack_start ( GTK_BOX ( hbox ),
     vscrollbar, FALSE, FALSE, 2 );
@@ -552,9 +584,28 @@ char *text;
 }
 
 
+static time_t do_round ( time_in, round_incr )
+time_t time_in;
+int round_incr;
+{
+  int t, n, r;
+
+  if ( round_incr == 0 )
+    return ( time_in );
+
+  t = (int) time_in;
+  r = t % round_incr;
+  t -= r;
+  if ( r > ( round_incr / 2 ) )
+    t += round_incr;
+
+  return ( (time_t) t );
+}
+
+
 
 static time_t summarize_day ( fp, seltasks, num_seltasks, then, type,
-  format, include_hours, include_annotations, is_last )
+  format, include_hours, include_annotations, round_incr, is_last )
 FILE *fp;
 ReportTaskData **seltasks;
 int num_seltasks;
@@ -563,6 +614,7 @@ report_type type;
 int format;
 int include_hours;
 int include_annotations;
+int round_incr;
 int is_last;
 {
   struct tm *tm;
@@ -572,7 +624,7 @@ int is_last;
   TaskTimeEntry *entry;
   TaskAnnotation **anns;
   int num_anns = 0;
-  int h, m, s;
+  int h, m, s, rounded;
   static char *weekdays[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri",
     "Sat" };
   time_t ret = 0;
@@ -585,7 +637,7 @@ int is_last;
       newline = "\n";
       break;
     case REPORT_OUTPUT_HTML:
-      newline = "<BR>\n";
+      newline = "<br>\n";
       break;
   }
 
@@ -637,7 +689,12 @@ int is_last;
       if ( anns )
         free ( anns );
     }
+    rounded = 0;
     if ( ( entry && entry->seconds ) || seltasks[loop]->todays_annotations ) {
+      rounded = do_round ( entry->seconds, round_incr );
+      h = rounded / 3600;
+      m = ( rounded - h * 3600 ) / 60;
+      s = rounded % 60;
       if ( type == REPORT_TYPE_DAILY ) {
         if ( ! found ) {
           switch ( format ) {
@@ -647,56 +704,59 @@ int is_last;
               fprintf ( fp, "-------------------------------------\n" );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TR><TH COLSPAN=\"%d\">", ncols + 1 );
+              fprintf ( fp, "<tr><th colspan=\"%d\">",
+                ncols + 1 );
               fprintf ( fp, "%02d/%02d/%d %s\n", mon, mday, year,
                 weekdays[wday] );
-              fprintf ( fp, "</TH></TR>\n" );
+              fprintf ( fp, "</th></tr>\n" );
               break;
           }
           found = 1;
         }
-        if ( entry && entry->seconds ) {
-          h = entry->seconds / 3600;
-          m = ( entry->seconds - h * 3600 ) / 60;
-          s = entry->seconds % 60;
-        } else {
-          h = m = s = 0;
-        }
         if ( format == REPORT_OUTPUT_HTML )
-          fprintf ( fp, "<TR>" );
+          fprintf ( fp, "<tr>" );
         if ( include_hours ) {
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s,
+              fprintf ( fp, "%3d:%02d:%02d - [%s] %s\n", h, m, s,
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
               fprintf ( fp,
-                "<TD ALIGN=\"right\" VALIGN=\"top\">%d:%02d:%02d</TD>",
+                "<td align=\"right\" valign=\"top\">%d:%02d:%02d</td>",
                 h, m, s );
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
         } else {
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%s\n", seltasks[loop]->taskdata->task->name );
+              fprintf ( fp, "[%s] %s\n",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
+                seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
         }
       }
-      if ( entry && entry->seconds ) {
-        ret += entry->seconds;
+      if ( rounded ) {
+        ret += rounded;
         /* update totals */
-        seltasks[loop]->weekly_total += entry->seconds;
-        seltasks[loop]->monthly_total += entry->seconds;
-        seltasks[loop]->yearly_total += entry->seconds;
-        seltasks[loop]->total += entry->seconds;
+        seltasks[loop]->weekly_total += rounded;
+        seltasks[loop]->monthly_total += rounded;
+        seltasks[loop]->yearly_total += rounded;
+        seltasks[loop]->total += rounded;
       }
       if ( seltasks[loop]->todays_annotations ) {
         switch ( format ) {
@@ -704,14 +764,14 @@ int is_last;
             fprintf ( fp, "%s", seltasks[loop]->todays_annotations );
             break;
           case REPORT_OUTPUT_HTML:
-            fprintf ( fp, "<TD>%s</TD>", seltasks[loop]->todays_annotations );
+            fprintf ( fp, "<td>%s</td>", seltasks[loop]->todays_annotations );
             break;
         }
         free ( seltasks[loop]->todays_annotations );
         seltasks[loop]->todays_annotations = NULL;
       }
       if ( type == REPORT_TYPE_DAILY && format == REPORT_OUTPUT_HTML )
-        fprintf ( fp, "</TR>\n" );
+        fprintf ( fp, "</tr>\n" );
     }
   }
 
@@ -725,9 +785,9 @@ int is_last;
         fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s, gettext("Total") );
         break;
       case REPORT_OUTPUT_HTML:
-        fprintf ( fp, "<TR><TD ALIGN=\"right\">%d:%02d:%02d</TD>",
+        fprintf ( fp, "<tr><td align=\"right\"><b>%d:%02d:%02d</b></td>",
           h, m, s );
-        fprintf ( fp, "<TD>%s</TD></TR>\n", gettext("Total") );
+        fprintf ( fp, "<td><b>%s</b></td></tr>\n", gettext("Total") );
         break;
     }
   }
@@ -747,11 +807,11 @@ int is_last;
               fprintf ( fp, "-------------------------------------\n" );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TR><TH COLSPAN=\"%d\">", ncols + 1 );
+              fprintf ( fp, "<tr><th colspan=\"%d\">", ncols + 1 );
               fprintf ( fp, "%s %s %s %02d/%02d/%d %s", gettext("Week"),
                 seltasks[loop]->week_start, gettext("to"), mon, mday, year,
                 weekdays[wday] );
-              fprintf ( fp, "</TH></TR>\n" );
+              fprintf ( fp, "</th></tr>\n" );
               break;
           }
           found = 1;
@@ -762,14 +822,18 @@ int is_last;
           s = seltasks[loop]->weekly_total % 60;
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s,
+              fprintf ( fp, "%3d:%02d:%02d - [%s] %s\n", h, m, s,
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
               fprintf ( fp,
-                "<TR><TD VALIGN=\"top\" ALIGN=\"right\">%d:%02d:%02d</TD>",
+                "<tr><td valign=\"top\" align=\"right\">%d:%02d:%02d</td>",
                 h, m, s );
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -777,10 +841,15 @@ int is_last;
         } else {
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%s\n", seltasks[loop]->taskdata->task->name );
+              fprintf ( fp, "[%s] %s\n",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
+                seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -791,14 +860,14 @@ int is_last;
               fprintf ( fp, "%s", seltasks[loop]->weekly_annotations );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD>%s</TD>", seltasks[loop]->weekly_annotations );
+              fprintf ( fp, "<td>%s</td>", seltasks[loop]->weekly_annotations );
               break;
           }
           free ( seltasks[loop]->weekly_annotations );
           seltasks[loop]->weekly_annotations = NULL;
         }
         if ( format == REPORT_OUTPUT_HTML )
-          fprintf ( fp, "</TR>\n" );
+          fprintf ( fp, "</tr>\n" );
       }
     }
   }
@@ -820,7 +889,7 @@ int is_last;
               fprintf ( fp, "-------------------------------------\n" );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TR><TH COLSPAN=\"%d\">%s %02d/%d</TH></TR>",
+              fprintf ( fp, "<tr><th colspan=\"%d\">%s %02d/%d</th></tr>",
                 ncols + 1, gettext("Month"), mon, year );
               break;
           }
@@ -832,13 +901,17 @@ int is_last;
           s = seltasks[loop]->monthly_total % 60;
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s,
+              fprintf ( fp, "%3d:%02d:%02d - [%s] %s\n", h, m, s,
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%d:%02d:%02d</TD>",
+              fprintf ( fp, "<td valign=\"top\">%d:%02d:%02d</td>",
                 h, m, s );
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -847,10 +920,15 @@ int is_last;
         else {
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%s\n", seltasks[loop]->taskdata->task->name );
+              fprintf ( fp, "[%s] %s\n",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
+                seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -861,7 +939,7 @@ int is_last;
               fprintf ( fp, "%s", seltasks[loop]->monthly_annotations );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD>%s</TD>",
+              fprintf ( fp, "<td>%s</td>",
                 seltasks[loop]->monthly_annotations );
               break;
           }
@@ -869,7 +947,7 @@ int is_last;
           seltasks[loop]->monthly_annotations = NULL;
         }
         if ( format == REPORT_OUTPUT_HTML )
-          fprintf ( fp, "</TR>\n" );
+          fprintf ( fp, "</tr>\n" );
       }
     }
   }
@@ -887,7 +965,7 @@ int is_last;
               fprintf ( fp, "-------------------------------------\n" );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TR><TH COLSPAN=\"%d\">%s %d</TH></TR>\n",
+              fprintf ( fp, "<tr><th colspan=\"%d\">%s %d</th></tr>\n",
                 ncols + 1, gettext("Year"), year );
               break;
           }
@@ -899,13 +977,17 @@ int is_last;
           s = seltasks[loop]->yearly_total % 60;
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s,
+              fprintf ( fp, "%3d:%02d:%02d - [%s] %s\n", h, m, s,
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%d:%02d:%02d</TD>",
+              fprintf ( fp, "<td valign=\"top\">%d:%02d:%02d</td>",
                 h, m, s );
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -914,10 +996,15 @@ int is_last;
         else {
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%s\n", seltasks[loop]->taskdata->task->name );
+              fprintf ( fp, "[%s] %s\n",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
+                seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -928,7 +1015,7 @@ int is_last;
               fprintf ( fp, "%s", seltasks[loop]->yearly_annotations );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD>%s</TD>",
+              fprintf ( fp, "<td>%s</td>",
                 seltasks[loop]->yearly_annotations );
               break;
           }
@@ -936,7 +1023,7 @@ int is_last;
           seltasks[loop]->yearly_annotations = NULL;
         }
         if ( format == REPORT_OUTPUT_HTML )
-          fprintf ( fp, "</TR>\n" );
+          fprintf ( fp, "</tr>\n" );
       }
     }
   }
@@ -952,7 +1039,7 @@ int is_last;
               fprintf ( fp, "-------------------------------------\n" );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TR><TH COLSPAN=\"%d\">%s</TH></TR>\n",
+              fprintf ( fp, "<tr><th COLSPAN=\"%d\">%s</th></tr>\n",
                 ncols + 1, gettext("Totals") );
               break;
           }
@@ -964,12 +1051,16 @@ int is_last;
           s = seltasks[loop]->total % 60;
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s,
+              fprintf ( fp, "%3d:%02d:%02d - [%s] %s\n", h, m, s,
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%d:%02d:%02d</TD>", h, m, s );
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">%d:%02d:%02d</td>", h, m, s );
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -978,10 +1069,15 @@ int is_last;
         else {
           switch ( format ) {
             case REPORT_OUTPUT_TEXT:
-              fprintf ( fp, "%s\n", seltasks[loop]->taskdata->task->name );
+              fprintf ( fp, "[%s] %s\n",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
+                seltasks[loop]->taskdata->task->name );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD VALIGN=\"top\">%s</TD>",
+              fprintf ( fp, "<td valign=\"top\">[%s] %s</td>",
+                seltasks[loop]->taskdata->task->project_id < 0 ?
+                "none" : seltasks[loop]->taskdata->project_name,
                 seltasks[loop]->taskdata->task->name );
               break;
           }
@@ -992,7 +1088,7 @@ int is_last;
               fprintf ( fp, "%s", seltasks[loop]->total_annotations );
               break;
             case REPORT_OUTPUT_HTML:
-              fprintf ( fp, "<TD>%s</TD>",
+              fprintf ( fp, "<td>%s</td>",
                 seltasks[loop]->total_annotations );
               break;
           }
@@ -1000,7 +1096,7 @@ int is_last;
           seltasks[loop]->total_annotations = NULL;
         }
         if ( format == REPORT_OUTPUT_HTML )
-          fprintf ( fp, "</TR>\n" );
+          fprintf ( fp, "</tr>\n" );
       }
     }
   }
@@ -1040,12 +1136,16 @@ char *text;
   char *command;
 
   path = get_client_path ( "netscape" );
+  if ( path == NULL )
+    path = get_client_path ( "mozilla" );
   if ( path == NULL ) {
     create_confirm_window ( CONFIRM_ERROR,
       gettext("Error"),
       gettext("Could not find \"netscape\"\nin your path."),
       gettext("Ok"), NULL, NULL,
       NULL, NULL, NULL, NULL );
+    free ( path );
+    return;
   }
 
   sprintf ( tempfile, "/tmp/gtimer-%d.html", (int) random () % 1000 );
@@ -1081,13 +1181,14 @@ gpointer data;
   int num_selected = 0;
   int range = REPORT_RANGE_TODAY;
   int format = REPORT_OUTPUT_TEXT;
+  int round_incr = REPORT_ROUND_NONE;
   FILE *fp;
   char *tempfile;
-  time_t now, time_start, time_end, time_loop;
+  time_t now, time_start, time_end, time_loop, total;
   struct tm *tm;
   struct stat buf;
   char *text = NULL;
-  int fd, ret, ncols;
+  int fd, ret, ncols, h, m, s;
 
   selected = GTK_LIST ( rd->task_list ) ->selection;
 
@@ -1164,6 +1265,14 @@ gpointer data;
     }
   }
 
+  /* which rounding ? */
+  for ( loop = 0; round_options[loop]; loop++ ) {
+    if ( GTK_OPTION_MENU ( rd->round_menu ) -> menu_item ==
+      rd->round_menu_items[loop] ) {
+      round_incr = round_values[loop];
+    }
+  }
+
   gtk_grab_remove ( rd->window );
   gtk_widget_destroy ( rd->window );
 
@@ -1200,6 +1309,14 @@ gpointer data;
       time_start = now - ONE_DAY * tm->tm_wday - ONE_DAY * 7;
       time_end = now - ONE_DAY * tm->tm_wday - ONE_DAY;
       break;
+    case REPORT_RANGE_THIS_AND_LAST_WEEK:
+      time_start = now - ONE_DAY * tm->tm_wday - ONE_DAY * 7;
+      time_end = time_start + ONE_DAY * 14;
+      break;
+    case REPORT_RANGE_LAST_TWO_WEEKS:
+      time_start = now - ONE_DAY * tm->tm_wday - ONE_DAY * 14;
+      time_end = now - ONE_DAY * tm->tm_wday - ONE_DAY;
+      break;
     case REPORT_RANGE_THIS_MONTH:
       time_start = now - ( ONE_DAY * ( tm->tm_mday - 1 ) );
       time_end = now;
@@ -1227,10 +1344,11 @@ gpointer data;
   }
 
   if ( format == REPORT_OUTPUT_HTML ) {
-    fprintf ( fp, "<HTML><HEAD><TITLE>GTimer %s</TITLE></HEAD>\n",
+    fprintf ( fp, "<html><head><title>GTimer %s</title></head>\n",
       gettext("Report") );
-    fprintf ( fp, "<BODY BGCOLOR=\"#FFFFFF\">\n" );
-    fprintf ( fp, "<TABLE BORDER=2>\n" );
+    fprintf ( fp, "%s", CSS_STYLE );
+    fprintf ( fp, "<body>\n" );
+    fprintf ( fp, "<table>\n" );
     ncols = 1;
     if ( rd->include_hours )
       ncols++;
@@ -1238,18 +1356,34 @@ gpointer data;
       ncols++;
   }
 
+  total = 0;
   for ( time_loop = time_start; time_loop <= time_end; time_loop += ONE_DAY ) {
-    summarize_day ( fp, seltasks, num_selected, time_loop, rd->type,
-      format, rd->include_hours, rd->include_annotations,
+    total += summarize_day ( fp, seltasks, num_selected, time_loop, rd->type,
+      format, rd->include_hours, rd->include_annotations, round_incr,
       ( time_loop == time_end ) );
   }
 
-  if ( format == REPORT_OUTPUT_HTML ) {
-    fprintf ( fp, "</TABLE><P><HR><FONT SIZE=\"-1\">%s ",
-      gettext("Generated by") );
-    fprintf ( fp, "<A HREF=\"%s\">GTimer v%s (%s)</A>.\n",
-      GTIMER_URL, GTIMER_VERSION, GTIMER_VERSION_DATE );
-    fprintf ( fp, "</BODY></HTML>\n" );
+  h = total / 3600;
+  m = ( total - h * 3600 ) / 60;
+  s = total % 60;
+
+  switch ( format ) {
+    case REPORT_OUTPUT_TEXT:
+      fprintf ( fp, "\n%s\n-------------------------------------\n",
+        gettext("Grand Total") );
+      fprintf ( fp, "%3d:%02d:%02d\n", h, m, s );
+      break;
+    case REPORT_OUTPUT_HTML:
+      fprintf ( fp, "<tr><th colspan=\"%d\">%s</th></tr>",
+        1 + ncols, gettext("Grand Total") );
+      fprintf ( fp, "<tr><td align=\"right\"><b>%d:%02d:%02d</b></td></tr>",
+        h, m, s );
+      fprintf ( fp, "</table><p><hr><font size=\"-1\">%s ",
+        gettext("Generated by") );
+      fprintf ( fp, "<a href=\"%s\">GTimer v%s (%s)</a>.\n",
+        GTIMER_URL, GTIMER_VERSION, GTIMER_VERSION_DATE );
+      fprintf ( fp, "</body></html>\n" );
+      break;
   }
 
   fclose ( fp );
@@ -1303,7 +1437,7 @@ GtkWidget *widget;
 gpointer data;
 {
   ReportData *rd = (ReportData *) data;
-#if GTK_VERSION < 10100
+#if OLD_GTK
   int loop;
 
   for ( loop = 0; loop < rd->num_tasks; loop++ )
@@ -1320,7 +1454,7 @@ GtkWidget *widget;
 gpointer data;
 {
   ReportData *rd = (ReportData *) data;
-#if GTK_VERSION < 10100
+#if OLD_GTK
   int loop;
 
   for ( loop = 0; loop < rd->num_tasks; loop++ )
@@ -1391,6 +1525,24 @@ ReportData *rd;
 }
 
 
+static GtkWidget *create_round_menu ( rd )
+ReportData *rd;
+{
+  GtkWidget *menu;
+  int loop;
+
+  menu = gtk_menu_new ();
+
+  for ( loop = 0; round_options[loop]; loop++ ) {
+    rd->round_menu_items[loop] =
+      gtk_menu_item_new_with_label ( round_options[loop] );
+    gtk_menu_append ( GTK_MENU ( menu ), rd->round_menu_items[loop] );
+    gtk_widget_show ( rd->round_menu_items[loop] );
+  }
+  return ( menu );
+}
+
+
 
 /*
 ** Create the report setup window.
@@ -1403,11 +1555,11 @@ report_type type;
   GtkWidget *table;
   /*GtkTooltips *tooltips;*/
   GtkWidget *label, *button, *time_menu, *output_menu,
-    *data_menu, *scrolled;
+    *data_menu, *round_menu, *scrolled;
   ReportData *rd;
   GList *items = NULL;
   int loop;
-  char msg[100];
+  char msg[100], temp[512];
 
   rd = (ReportData *) malloc ( sizeof ( ReportData ) );
   memset ( rd, '\0', sizeof ( ReportData ) );
@@ -1422,7 +1574,7 @@ report_type type;
   gdk_window_set_icon ( GTK_WIDGET ( rd->window )->window,
     NULL, appicon2, appicon2_mask );
 
-  table = gtk_table_new ( 3, 2, FALSE );
+  table = gtk_table_new ( 4, 2, FALSE );
   gtk_table_set_row_spacings (GTK_TABLE (table), 4);
   gtk_table_set_col_spacings (GTK_TABLE (table), 8);
   gtk_container_border_width (GTK_CONTAINER (table), 6);
@@ -1466,6 +1618,18 @@ report_type type;
   gtk_table_attach_defaults ( GTK_TABLE (table), rd->data_menu, 1, 2, 2, 3 );
   gtk_widget_show ( rd->data_menu );
 
+  label = gtk_label_new ( gettext("Rounding: ") );
+  gtk_table_attach_defaults ( GTK_TABLE (table), label, 0, 1, 3, 4 );
+  gtk_widget_show ( label );
+
+  rd->round_menu = gtk_option_menu_new ();
+  round_menu = create_round_menu ( rd );
+  gtk_option_menu_set_menu ( GTK_OPTION_MENU ( rd->round_menu ), round_menu );
+  gtk_option_menu_set_history ( GTK_OPTION_MENU ( rd->round_menu ),
+    REPORT_OUTPUT_TEXT );
+  gtk_table_attach_defaults ( GTK_TABLE (table), rd->round_menu, 1, 2, 3, 4 );
+  gtk_widget_show ( rd->round_menu );
+
   gtk_widget_show ( table );
 
   label = gtk_label_new ( gettext("Tasks to include:") );
@@ -1490,14 +1654,21 @@ report_type type;
     num_visible_tasks );
   rd->num_tasks = num_visible_tasks;
   for ( loop = 0; loop < num_visible_tasks; loop++ ) {
+    if ( visible_tasks[loop]->task->project_id < 0 )
+      snprintf ( temp, sizeof ( temp ), "%s",
+         visible_tasks[loop]->task->name );
+    else
+      snprintf ( temp, sizeof ( temp ),
+        "[%s] %s", visible_tasks[loop]->project_name,
+        visible_tasks[loop]->task->name );
     rd->list_items[loop] =
-      gtk_list_item_new_with_label ( visible_tasks[loop]->task->name );
+      gtk_list_item_new_with_label ( temp );
     items = g_list_append ( items, rd->list_items[loop] );
     rd->tasks[loop] = visible_tasks[loop];
     gtk_widget_show ( rd->list_items[loop] );
   }
 
-#if GTK_VERSION < 10100
+#if OLD_GTK
   gtk_container_add (GTK_CONTAINER (scrolled), rd->task_list );
 #else
   gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW ( scrolled ),
@@ -1561,43 +1732,4 @@ report_type type;
 
 
 
-/*
-** Determine the path to an executable using the $PATH environment
-** variable.
-** Return value should be freed (unless it's NULL!)
-*/
-static char *get_client_path ( file )
-char *file;
-{
-  char *path;
-  static char *path2 = "/usr/bin/X11:/usr/local/bin:/usr/bin";
-  char *env;
-  struct stat buf;
-  char *ret, *ptr;
-
-  env = getenv ( "PATH" );
-  if ( env ) {
-    path = (char *) malloc ( strlen ( env ) + strlen ( path2 ) + 2 );
-    sprintf ( path, "%s:%s", env, path2 );
-  } else {
-    path = (char *) malloc ( strlen ( path2 ) + 1 );
-    strcpy ( path, path2 );
-  }
-
-  ptr = strtok ( path, ":" );
-  while ( ptr ) {
-    ret = (char *) malloc ( strlen ( ptr ) + strlen ( file ) + 2 );
-    sprintf ( ret, "%s/%s", ptr, file );
-    if ( stat ( ret, &buf ) == 0 ) {
-      /* found it */
-      free ( path );
-      return ( ret );
-    }
-    ptr = strtok ( NULL, ":" );
-  }
-
-  /* not found */
-  free ( path );
-  return ( NULL );
-}
 
