@@ -2,7 +2,7 @@
  * Routines for dealing with tasks
  *
  * Copyright:
- *	(C) 1999 Craig Knudsen, cknudsen@radix.net
+ *	(C) 1999 Craig Knudsen, cknudsen@cknudsen.com
  *	See accompanying file "COPYING".
  * 
  *	This program is free software; you can redistribute it and/or
@@ -332,10 +332,11 @@ char *taskdir;
     return ( TASK_ERROR_SYSTEM_ERROR );
   }
 
-  fprintf ( fp, "Format: 1.1\n" );
+  fprintf ( fp, "Format: 1.2\n" );
   fprintf ( fp, "Name: %s\n", task->name );
   fprintf ( fp, "Created: %u\n", (unsigned int)task->created );
   fprintf ( fp, "Options: %u\n", task->options );
+  fprintf ( fp, "Project: %d\n", task->project_id );
   fprintf ( fp, "Data:\n" );
 
   for ( loop = 0; loop < task->num_entries; loop++ ) {
@@ -407,7 +408,7 @@ Task **task;
   int fd;
   Task *newtask;
   char line[512], *ptr, *ptr2, temp[10], *annfile, *anntext;
-  int len, created, number, options;
+  int len, created, number, options, project_id = -1;
   TaskTimeEntry *entry;
   TaskAnnotation *a;
   struct stat buf;
@@ -425,7 +426,8 @@ Task **task;
   len = strlen ( line );
   if ( line[len-1] == '\n' )
     line[len-1] = '\0';
-  if ( strcmp ( line, "Format: 1.0" ) && strcmp ( line, "Format: 1.1" ) ) {
+  if ( strcmp ( line, "Format: 1.0" ) && strcmp ( line, "Format: 1.1" ) &&
+    strcmp ( line, "Format: 1.2" ) ) {
     fclose ( fp );
     return ( TASK_ERROR_BAD_FILE );
   }
@@ -433,6 +435,7 @@ Task **task;
   newtask = (Task *) malloc ( sizeof ( Task ) );
   memset ( newtask, '\0', sizeof ( Task ) );
   newtask->number = number;
+  newtask->project_id = -1; /* no project */
 
   while ( fgets ( line, 512, fp ) ) {
     len = strlen ( line );
@@ -447,6 +450,9 @@ Task **task;
     } else if ( strncmp ( line, "Created:", 8 ) == 0 ) {
       sscanf ( line + 8, "%d", &created );
       newtask->created = (time_t) created;
+    } else if ( strncmp ( line, "Project:", 8 ) == 0 ) {
+      sscanf ( line + 8, "%d", &project_id );
+      newtask->project_id = project_id;
     } else if ( strncmp ( line, "Options:", 8 ) == 0 ) {
       sscanf ( line + 8, "%d", &options );
       newtask->options = (unsigned int) options;
