@@ -2,7 +2,7 @@
  * GTimer
  *
  * Copyright:
- *	(C) 1999 Craig Knudsen, cknudsen@radix.net
+ *	(C) 1999 Craig Knudsen, cknudsen@cknudsen.com
  *	See accompanying file "COPYING".
  * 
  *	This program is free software; you can redistribute it and/or
@@ -24,10 +24,10 @@
  *	Helps you keep track of time spent on different tasks.
  *
  * Author:
- *	Craig Knudsen, cknudsen@radix.net, http://www.radix.net/~cknudsen
+ *	Craig Knudsen, cknudsen@cknudsen.com, http://www.cknudsen.com
  *
  * Home Page:
- *	http://www.radix.net/~cknudsen/gtimer/
+ *	http://www.cknudsen.com/gtimer/
  *
  * History:
  *	09-Mar-2000	Updated call to create_confirm_window()
@@ -59,6 +59,7 @@
 #define gettext(a)      a
 #endif
 
+#include "project.h"
 #include "task.h"
 #include "gtimer.h"
 #include "config.h"
